/*
	CreateSpatialScoreCmd.cc

	Will 18, 2001, 2006
*/

#include "OpResources.h"
#include "CreateSpatialScoreCmd.h"
#include "AppWindow.h"
#include <WorkingDialogManager.h>
#include <FileAccess.h>


CreateSpatialScoreCmd::CreateSpatialScoreCmd ( char *name, int active, unsigned int waveNum, SpatialPak *spatialPak ) : BreakCmd ( name, active )
{
	_waveNum = waveNum;
	_spatialPak = spatialPak;
	_scoreSpatialPak = NULL;
	_error[0] = 0;
	_totalSamples = (*wavePak)[waveNum-1].pWave->totalSamples();
	_sampleRate = (*wavePak)[waveNum-1].pWave->sampleRate();
	_percentDone = 0;
	_target = NULL;
	_skipTime = 0;	// in case processing part of wave required
}


CreateSpatialScoreCmd::~CreateSpatialScoreCmd()
{
	if ( _target != NULL )
		delete _target;
		
	if ( _scoreSpatialPak != NULL )
		postCreate();
}


void CreateSpatialScoreCmd::doCommand()
{
	updateTimeout(350);	/* progress bar update, msec - no actual progress */
	snprintf ( _formatString, sizeof ( _formatString ), " %s%d%s%d%s",
				"%.", mixwaveResources->fpDigits(), "f %.",
				mixwaveResources->fpDigits(), "f" );
	_formatString [ sizeof ( _formatString ) - 1 ] = 0;

	if ( preCreate() )
	{
		_done = True;
		postCreate();
		return;
	}

	writeHeader();

	char buf[BUFSIZE];
	char num[64];
	
	strncpy ( buf, "i1", sizeof ( buf ) );
	buf [ sizeof ( buf ) - 1 ] = 0;
	
	float startTime = 0;
	float duration;

	if ( (*wavePak)[_waveNum-1].startSel != (*wavePak)[_waveNum-1].endSel )
	{
		_skipTime = ( float ) (*wavePak)[_waveNum-1].startSel / ( float ) _sampleRate;
		duration = ( float ) ( (*wavePak)[_waveNum-1].endSel - (*wavePak)[_waveNum-1].startSel ) / ( float ) _sampleRate;	}
	else
		duration = ( float ) _totalSamples / ( float ) _sampleRate;

	snprintf ( num, sizeof ( num ), _formatString, startTime, duration );
	num [ sizeof ( num ) - 1 ] = 0;
	strncat ( buf, num, sizeof ( buf ) - strlen ( buf ) - 1 );
	buf [ sizeof ( buf ) - 1 ] = 0;

	snprintf ( num, sizeof ( num ), " %u %u", _scoreSpatialPak->panType, _scoreSpatialPak->filterType );
	num [ sizeof ( num ) - 1 ] = 0;
	strncat ( buf, num, sizeof ( buf ) - strlen ( buf ) - 1 );
	buf [ sizeof ( buf ) - 1 ] = 0;

	snprintf ( num, sizeof ( num ), _formatString, _scoreSpatialPak->angularSep, _scoreSpatialPak->signalScale );
	num [ sizeof ( num ) - 1 ] = 0;
	strncat ( buf, num, sizeof ( buf ) - strlen ( buf ) - 1 );
	buf [ sizeof ( buf ) - 1 ] = 0;

	char formatString[64];
	snprintf ( formatString, sizeof ( formatString ), " %s%d%s",
				"%.", mixwaveResources->fpDigits(), "f " );
	formatString [ sizeof ( formatString ) - 1 ] = 0;

	snprintf ( num, sizeof ( num ), formatString, _scoreSpatialPak->detune );
	num [ sizeof ( num ) - 1 ] = 0;
	strncat ( buf, num, sizeof ( buf ) - strlen ( buf ) - 1 );
	buf [ sizeof ( buf ) - 1 ] = 0;
	
	snprintf ( num, sizeof ( num ), " %u", _scoreSpatialPak->phaser );
	num [ sizeof ( num ) - 1 ] = 0;
	strncat ( buf, num, sizeof ( buf ) - strlen ( buf ) - 1 );
	buf [ sizeof ( buf ) - 1 ] = 0;

	strncat ( buf, "\ni99 ", sizeof ( buf ) );
	buf [ sizeof ( buf ) - 1 ] = 0;

	snprintf ( num, sizeof ( num ), _formatString, startTime, duration + _scoreSpatialPak->rvbTailTime );
	num [ sizeof ( num ) - 1 ] = 0;
	strncat ( buf, num, sizeof ( buf ) - strlen ( buf ) - 1 );
	buf [ sizeof ( buf ) - 1 ] = 0;

	snprintf ( num, sizeof ( num ), " %u %u", _scoreSpatialPak->panLType, _scoreSpatialPak->panRType );
	num [ sizeof ( num ) - 1 ] = 0;
	strncat ( buf, num, sizeof ( buf ) - strlen ( buf ) - 1 );
	buf [ sizeof ( buf ) - 1 ] = 0;

	snprintf ( num, sizeof ( num ), _formatString, _scoreSpatialPak->delayL, _scoreSpatialPak->delayR );
	num [ sizeof ( num ) - 1 ] = 0;
	strncat ( buf, num, sizeof ( buf ) - strlen ( buf ) - 1 );
	buf [ sizeof ( buf ) - 1 ] = 0;

	snprintf ( num, sizeof ( num ), " %u %u", _scoreSpatialPak->filterLType, _scoreSpatialPak->filterRType );
	num [ sizeof ( num ) - 1 ] = 0;
	strncat ( buf, num, sizeof ( buf ) - strlen ( buf ) - 1 );
	buf [ sizeof ( buf ) - 1 ] = 0;

	if ( _scoreSpatialPak->rvbTailTime > 0 )
		strncat ( buf, " 1\n", sizeof ( buf ) - strlen ( buf ) - 1 );
	else
		strncat ( buf, " 0\n", sizeof ( buf ) - strlen ( buf ) - 1 );

	buf[ sizeof ( buf ) - 1 ] = 0;

	_target->writeF ( buf );

	if ( _target->errNum() != 0 )
	{
		strncpy ( _error, _target->error(), sizeof ( _error ) );
		_error [ sizeof ( _error ) - 1 ] = 0;
	}

	writeFooter();
	writeOrchFile();
	postCreate();
	_done = True;	// since we only have 1 pass, doesn't need to be an BreakCmd subclass really...
}


int CreateSpatialScoreCmd::preCreate()
{
	_scoreSpatialPak = new SpatialPak;

	// take fixed values from original spatial template pak
	_scoreSpatialPak->panType = _spatialPak->panType;
	_scoreSpatialPak->filterType = _spatialPak->filterType;
	_scoreSpatialPak->angularSep = _spatialPak->angularSep;
	_scoreSpatialPak->signalScale = _spatialPak->signalScale;
	_scoreSpatialPak->detune = _spatialPak->detune;
	_scoreSpatialPak->phaser = _spatialPak->phaser;
	_scoreSpatialPak->rvbTailTime = _spatialPak->rvbTailTime;
	_scoreSpatialPak->panLType = _spatialPak->panLType;
	_scoreSpatialPak->panRType = _spatialPak->panRType;
	_scoreSpatialPak->delayL = _spatialPak->delayL;
	_scoreSpatialPak->delayR = _spatialPak->delayR;
	_scoreSpatialPak->filterLType = _spatialPak->filterLType;
	_scoreSpatialPak->filterRType = _spatialPak->filterRType;

	return ( 0 );
}


void CreateSpatialScoreCmd::postCreate()
{
	if ( _scoreSpatialPak != NULL )
	{
		delete _scoreSpatialPak;
		_scoreSpatialPak = NULL;
	}
}


void CreateSpatialScoreCmd::writeHeader()
{
	char scoreFile [ strlen ( _spatialPak->name ) + strlen ( mixwaveResources->cspatialScoreSuffix() ) + 1 ];

	strncpy ( scoreFile, _spatialPak->name, sizeof ( scoreFile ) );
	scoreFile [ sizeof ( scoreFile ) - 1 ] = 0;

	strncat ( scoreFile, mixwaveResources->cspatialScoreSuffix(), sizeof ( scoreFile ) - strlen ( scoreFile ) - 1 );
	scoreFile [ sizeof ( scoreFile ) - 1 ] = 0;

	_target = new FileAccess ( scoreFile, "w" );

	char buf [ strlen ( scoreFile ) + 6 ];
	
	strcpy ( buf, "; " );
	strncat ( buf, scoreFile, sizeof ( buf ) - strlen ( buf ) - 1 );
	strncat ( buf, "\n;\n", sizeof ( buf ) - strlen ( buf ) - 1 );

	_target->writeF ( buf );
	
	if ( _target->errNum() != 0 )
	{
		strncpy ( _error, _target->error(), sizeof ( _error ) );
		_error [ sizeof ( _error ) - 1 ] = 0;
	}
}


void CreateSpatialScoreCmd::writeFooter()
{
	char scoreFile [ strlen ( _spatialPak->name ) + strlen ( mixwaveResources->cspatialScoreSuffix() ) + 1 ];

	strncpy ( scoreFile, _spatialPak->name, sizeof ( scoreFile ) );
	scoreFile [ sizeof ( scoreFile ) - 1 ] = 0;

	strncat ( scoreFile, mixwaveResources->cspatialScoreSuffix(), sizeof ( scoreFile ) - strlen ( scoreFile ) - 1 );
	scoreFile [ sizeof ( scoreFile ) - 1 ] = 0;

	char buf[8];
	
	strncpy ( buf, ";\ne\n;\n", sizeof ( buf ) );
	buf [ sizeof ( buf ) - 1 ] = 0;

	_target->writeF ( buf );
	
	if ( _target->errNum() != 0 )
	{
		strncpy ( _error, _target->error(), sizeof ( _error ) );
		_error [ sizeof ( _error ) - 1 ] = 0;
	}
	
	delete _target;
	_target = NULL;
}


void CreateSpatialScoreCmd::writeOrchFile()
{
	FileAccess *target;
	float pitchval;
	char buf[BUFSIZE * 10];
	char num[64];
	char pitch[64];
	char orchFile [ strlen ( _spatialPak->name ) + strlen ( mixwaveResources->cspatialOrchSuffix() ) + 1 ];

	strncpy ( orchFile, _spatialPak->name, sizeof ( orchFile ) );
	orchFile [ sizeof ( orchFile ) - 1 ] = 0;

	strncat ( orchFile, mixwaveResources->cspatialOrchSuffix(), sizeof ( orchFile ) - strlen ( orchFile ) - 1 );
	orchFile [ sizeof ( orchFile ) - 1 ] = 0;

	target = new FileAccess ( orchFile, "w" );
	
	if ( target->errNum() != 0 )
	{
		strncpy ( _error, _target->error(), sizeof ( _error ) );
		_error [ sizeof ( _error ) - 1 ] = 0;
	}
		
	snprintf ( buf, sizeof ( buf ), "; %s\n\n", orchFile );
	buf [ sizeof ( buf ) - 1 ] = 0;
	strncat ( buf, "; 'csound -W -o <out.wav> <name.orc> <name.sco>'\n\n", sizeof ( buf ) - strlen ( buf ) - 1 );
	buf [ sizeof ( buf ) - 1 ] = 0;
	strncat ( buf, "#define PI # 3.14159265359 #\n#define ROOT2 # 1.414213562 #\
\n#define SVEL # 231.68 #\n\n", sizeof ( buf ) - strlen ( buf ) - 1 );
	buf [ sizeof ( buf ) - 1 ] = 0;
	target->writeF ( buf );
	
	snprintf ( buf, sizeof ( buf ), "sr = %ld\n", (*wavePak)[_waveNum-1].pWave->sampleRate() );
	buf [ sizeof ( buf ) - 1 ] = 0;
	target->writeF ( buf );

	snprintf ( buf, sizeof ( buf ), "kr = %u\n", mixwaveResources->kRate() );
	buf [ sizeof ( buf ) - 1 ] = 0;
	target->writeF ( buf );

	snprintf ( buf, sizeof ( buf ), "ksmps = %ld\n",
				(*wavePak)[_waveNum-1].pWave->sampleRate() / mixwaveResources->kRate() );
	buf [ sizeof ( buf ) - 1 ] = 0;
	target->writeF ( buf );

	char formatString[64];
	snprintf ( formatString, sizeof ( formatString ), "%s%d%s",
				"%.", mixwaveResources->fpDigits(), "f" );
	formatString [ sizeof ( formatString ) - 1 ] = 0;

	// pitchval is ratio of template size to file size (smps), used to
	// obtain interpolated template values and compensated skiptime
	pitchval = ( float ) mixwaveResources->defaultTemplateSize() / (float ) (*wavePak)[_waveNum-1].pWave->totalSamples();
	snprintf ( pitch, sizeof ( pitch ), formatString, pitchval );
	pitch [ sizeof ( pitch ) - 1 ] = 0;

	snprintf ( num, sizeof ( num ), formatString, _skipTime );
	num [ sizeof ( num ) - 1 ] = 0;

	strncpy ( buf, "nchnls = 2\n\ngarvbsig init 0\n\ninstr 1; direct signal\n\nipanfn = \
p4\nifiltfn = p5\niskiptime = ", sizeof ( buf ) );
	buf [ sizeof ( buf ) - 1 ] = 0;
	strncat ( buf, num, sizeof ( buf ) - strlen ( buf ) - 1 );
	buf [ sizeof ( buf ) - 1 ] = 0;
	strncat ( buf, "\nkemit = p6\nkampscl = p7\nidetune = p8\niphasectl = \
p9\n\n", sizeof ( buf )  - strlen ( buf ) - 1 );
	buf [ sizeof ( buf ) - 1 ] = 0;
	target->writeF ( buf );

	snprintf ( num, sizeof ( num ), formatString, _skipTime * pitchval );
	num [ sizeof ( num ) - 1 ] = 0;

	snprintf ( buf, sizeof ( buf ), "arvbsig diskin \"%s_reverb.flt\", %s, %s, 0, 6\n\
arangesig diskin \"%s_range.flt\", %s, %s, 0, 6\nabwsig diskin \"%s_bwidth.flt\", %s, %s, 0, 6\n\
asweepsig diskin \"%s_sweep.flt\", %s, %s, 0, 6\napansig diskin \"%s_pan.flt\", %s, %s, 0, 6\n\
adplrsig diskin \"%s_dplr.flt\", %s, %s, 0, 6\n\n",
				_spatialPak->name, pitch, num, _spatialPak->name, pitch, num, _spatialPak->name, pitch, num,
				_spatialPak->name, pitch, num, _spatialPak->name, pitch, num, _spatialPak->name, pitch, num );
	buf [ sizeof ( buf ) - 1 ] = 0;
	target->writeF ( buf );
	
	strncpy ( buf, "kdplr downsamp adplrsig\nksweep downsamp asweepsig\n\
kbw downsamp abwsig\n\n", 74 );
	buf [ sizeof ( buf ) - 1 ] = 0;
	target->writeF ( buf );

	strncpy ( buf, "ie = 0.075\n\n", sizeof ( buf ) );
	buf [ sizeof ( buf ) - 1 ] = 0;
	target->writeF ( buf );

	strncpy ( buf, "ay = sin (apansig * $PI)\nax = cos (apansig * $PI)\n\n\
apath1 = sqrt(((ax-ie) * (ax-ie)) + (ay * ay))\n\
apath2 = sqrt(((ax+ie) * (ax+ie)) + (ay * ay))\n\nadiff = apath1 - apath2\n\n", sizeof ( buf ) );
	buf [ sizeof ( buf ) - 1 ] = 0;
	target->writeF ( buf );

	strncpy ( buf, "adly = adiff / $SVEL\nadly = sqrt(adly * adly)\n\n", sizeof ( buf ) );
	buf [ sizeof ( buf ) - 1 ] = 0;
	target->writeF ( buf );

	strncpy ( buf, "asiga diskin ", sizeof ( buf ) );
	buf [ sizeof ( buf ) - 1 ] = 0;
	target->writeF ( buf );

	snprintf ( buf, sizeof ( buf ), "\"%s\", ", (*wavePak)[_waveNum-1].pWave->name() );
	buf [ sizeof ( buf ) - 1 ] = 0;

	strncat ( buf, "kdplr, iskiptime\nasig = asiga\nasigb = 0\nasigc = 0\n\nif p9 = 0 \
goto endcascade\n", sizeof ( buf ) );
	buf [ sizeof ( buf ) - 1 ] = 0;
	target->writeF ( buf );

	strncpy ( buf, "if kdplr = 1 goto endcascade\n\
adetune = (adplrsig * (abs(arangesig))) / idetune\nkdetune downsamp adetune\n\
asigb diskin ", sizeof ( buf ) );
	buf [ sizeof ( buf ) - 1 ] = 0;
	target->writeF ( buf );

	snprintf ( buf, sizeof ( buf ), "\"%s\", kdplr+kdetune, iskiptime\n",
				(*wavePak)[_waveNum-1].pWave->name() );
	buf [ sizeof ( buf ) - 1 ] = 0;
	target->writeF ( buf );

	snprintf ( buf, sizeof ( buf ), "asigc diskin \"%s\", kdplr-kdetune, iskiptime\n",
				(*wavePak)[_waveNum-1].pWave->name() );
	buf [ sizeof ( buf ) - 1 ] = 0;
	strncat ( buf, "asig = asiga + asigb/3 + asigc/3\nasig = asig * 3/5\n\
endcascade:\n\n", sizeof ( buf ) - strlen ( buf ) - 1 );
	buf [ sizeof ( buf ) - 1 ] = 0;
	target->writeF ( buf );

	strncpy ( buf, "asig = (1 / arangesig) * asig\n\nif (ksweep = 0) \
goto skipfilter\n\n", sizeof ( buf ) );
	buf [ sizeof ( buf ) - 1 ] = 0;
	target->writeF ( buf );

	strncpy ( buf, "if ifiltfn = 1 goto notch\nasig reson asig, ksweep, kbw, 1\n\
goto skipfilter\n\nnotch:\nasig1 tone asig, ksweep-kbw/2\n\
asig2 atone asig, ksweep+kbw/2\nasig = asig1+asig2\n\n\
skipfilter:\n\n", sizeof ( buf ) );
	buf [ sizeof ( buf ) - 1 ] = 0;
	target->writeF ( buf );

	strncpy ( buf, "asigd  vdelay asig, adly*1000, 29\n\n", sizeof ( buf ) );
	buf [ sizeof ( buf ) - 1 ] = 0;
	target->writeF ( buf );

	strncpy ( buf, "if ipanfn = 1 goto pan1\nif ipanfn = 2 goto pan2\n\n\
a1 = apansig * $PI / 2\na2 = cos ( a1 )\n\
a3 = sin ( a1 )\ngoto endpan\n\npan1:\na1 = apansig * $PI\na2 = sqrt ( 2 ) \
/ 2 * ( cos ( a1 - $PI / 2 ) - sin ( a1 - $PI / 2 ) )\na3 = sqrt ( 2 ) / 2 \
* ( cos ( a1 - $PI / 2 ) + sin ( a1 - $PI / 2 ) )\ngoto endpan\n\n", sizeof ( buf ) );
	buf [ sizeof ( buf ) - 1 ] = 0;
	target->writeF ( buf );

	strncpy ( buf, "pan2:\na1 = apansig * $PI\na2 = 1/sqrt(2)*(cos(a1)+sin(a1))\n\
a3 = 1/sqrt(2)*(cos(a1)-sin(a1))\n\nendpan:\n\n", sizeof ( buf ) );
	buf [ sizeof ( buf ) - 1 ] = 0;
	target->writeF ( buf );

	strncpy ( buf, "apath1 = asig\napath2 = asig\nkdiff downsamp adiff\n", sizeof ( buf ) );
	buf [ sizeof ( buf ) - 1 ] = 0;
	strncat ( buf, "if kdiff = 0 kgoto endxpos\n\nif kdiff \
< 0 kgoto objectright\napath2=asigd\nkgoto endxpos\nobjectright:\n\
apath1 = asigd\nendxpos:\n\n", sizeof ( buf ) - strlen ( buf ) - 1 );
	buf [ sizeof ( buf ) - 1 ] = 0;
	target->writeF ( buf );

	strncpy ( buf, "aoutl = apath1 * a2 * kampscl\naoutr = apath2 * a3 \
* kampscl\n\nouts aoutl, aoutr\n\n",
	sizeof ( buf ) );
	buf [ sizeof ( buf ) - 1 ] = 0;
	target->writeF ( buf );

	strncpy ( buf, "garvbsig = garvbsig + asig * arvbsig * kampscl\n\
endin\n\n\n", sizeof ( buf ) );
	buf [ sizeof ( buf ) - 1 ] = 0;
	target->writeF ( buf );
	
	// if no reverb tail time, skip this altogether
	strncpy ( buf, "instr 99; global reverb unit - reflections\n\nif (p10 == 0) goto \
skipreverb\n\n", sizeof ( buf ) );
	buf [ sizeof ( buf ) - 1 ] = 0;
	strncat ( buf, "ipanfnl = p4\nipanfnr = p5\nidelayl = p6\n\
idelayr = p7\nifiltfnl = p8\nifiltfnr = p9\n\n", sizeof ( buf ) - strlen ( buf ) - 1 );
	buf [ sizeof ( buf ) - 1 ] = 0;
	target->writeF ( buf );

	snprintf ( buf, sizeof ( buf ), "arvblsig diskin \"%s_rvbl.flt\", %s, %s, 0, 6\n\
arvbrsig diskin \"%s_rvbr.flt\", %s, %s, 0, 6\napanlsig diskin \"%s_panl.flt\", %s, %s, 0, 6\n\
apanrsig diskin \"%s_panr.flt\", %s, %s, 0, 6\nabwlsig diskin \"%s_bwl.flt\", %s, %s, 0, 6\n\
abwrsig diskin \"%s_bwr.flt\", %s, %s, 0, 6\nasweeplsig diskin \"%s_swpl.flt\", %s, %s, 0, 6\n\
asweeprsig diskin \"%s_swpr.flt\", %s, %s, 0, 6\naamplsig diskin \"%s_ampl.flt\", %s, %s, 0, 6\n\
aamprsig diskin \"%s_ampr.flt\", %s, %s, 0, 6\n\n",
				_spatialPak->name, pitch, num, _spatialPak->name, pitch, num, _spatialPak->name, pitch, num,
				_spatialPak->name, pitch, num, _spatialPak->name, pitch, num, _spatialPak->name, pitch, num,
				_spatialPak->name, pitch, num, _spatialPak->name, pitch, num, _spatialPak->name, pitch, num,
				_spatialPak->name, pitch, num );
	buf [ sizeof ( buf ) - 1 ] = 0;
	target->writeF ( buf );
	
	strncpy ( buf, "krvbl downsamp arvblsig\nkrvbr downsamp arvbrsig\n\
kbwl downsamp abwlsig\nkbwr downsamp abwrsig\nksweepl downsamp asweeplsig\n\
ksweepr downsamp asweeprsig\n\n", sizeof ( buf ) );
	buf [ sizeof ( buf ) - 1 ] = 0;
	target->writeF ( buf );

	strncpy ( buf, "if ipanfnl = 1 goto pan2l\n\na1l = apanlsig * $PI / \
2\na2l = cos ( a1l )\na3l = sin ( a1l )\ngoto endpanl\n\npan2l:\na1l = \
apanlsig * $PI\na2l = sqrt ( 2 ) / 2 * ( cos ( a1l - $PI / 2 ) - sin ( \
a1l - $PI / 2 ) )\na3l = sqrt ( 2 ) / 2 * ( cos ( a1l - $PI / 2 ) + sin \
( a1l - $PI / 2 ) )\n\n", sizeof ( buf ) );
	buf [ sizeof ( buf ) - 1 ] = 0;
	target->writeF ( buf );

	strncpy ( buf, "endpanl:\n\nif ipanfnr = 1 goto \
pan2r\n\na1r = apanrsig * $PI / 2\na2r = cos ( a1r )\na3r = sin ( a1r )\n\
goto endpanr\n\npan2r:\na1r = apanrsig * $PI\n\
a2r = sqrt ( 2 ) / 2 * ( cos ( a1r - $PI / 2 ) - sin ( a1r - $PI / 2 ) )\n\
a3r = sqrt ( 2 ) / 2 * ( cos ( a1r - $PI / 2 ) + sin ( a1r - $PI / 2 ) )\n\
endpanr:\n\n", sizeof ( buf ) );
	buf [ sizeof ( buf ) - 1 ] = 0;
	target->writeF ( buf );

	strncpy ( buf, "asigl = garvbsig\n\nif (kbwl == 0) goto \
endfiltl\nif ifiltfnl = 1 goto notchl\nasigl reson garvbsig, ksweepl, \
kbwl, 1\ngoto endfiltl\n\nnotchl:\nasigl1 tone garvbsig, ksweepl-kbwl/2\n\
asigl2 atone garvbsig, ksweepl+kbwl/2\nasigl = asigl1+asigl2\n\n\
endfiltl:\n\n", sizeof ( buf ) );
	buf [ sizeof ( buf ) - 1 ] = 0;
	target->writeF ( buf );

	strncpy ( buf, "asigr = garvbsig\nif (kbwr == 0) goto \
endfiltr\nif ifiltfnr = 1 goto notchr\nasigr reson garvbsig, ksweepr, \
kbwr, 1\ngoto endfiltr\n\nnotchr:\nasigr1 tone garvbsig, ksweepr-kbwr/2\n\
asigr2 atone garvbsig, ksweepr+kbwr/2\nasigr = asigr1+asigr2\n\n\
endfiltr:\n\n; block reverb\naoutl reverb asigl, krvbl\n\
aoutr reverb asigr, krvbr\n\n",
	sizeof ( buf ) );
	buf [ sizeof ( buf ) - 1 ] = 0;
	target->writeF ( buf );

	strncpy ( buf, "asigl = aoutl\n\nif (p6 == 0) goto nodlyl\nasigl delay \
aoutl, idelayl\nnodlyl: \n\nasigr = aoutr\n\nif (p7 == 0) goto nodlyr\n\
asigr delay aoutr, idelayr\nnodlyr:\n\n", sizeof ( buf ) );
	buf [ sizeof ( buf ) - 1 ] = 0;
	target->writeF ( buf );

	strncpy ( buf, "ie = .075\n\nay = sin (apanlsig * $PI)\nax = cos \
(apanlsig * $PI)\n\napath1 = sqrt(((ax-ie) * (ax-ie)) + (ay * ay))\napath2 = \
sqrt(((ax+ie) * (ax+ie)) + (ay * ay))\n\n", sizeof ( buf ) );
	buf [ sizeof ( buf ) - 1 ] = 0;
	target->writeF ( buf );

	strncpy ( buf, "adiffl = apath1 - apath2\n\nadlyl = adiffl / \
231.68\nadlyl = sqrt(adlyl * adlyl)\n\nay = \
sin (apanrsig * $PI)\nax = cos (apanrsig * $PI)\n\napath1 = sqrt(((ax-ie) * \
(ax-ie)) + (ay * ay))\napath2 = sqrt(((ax+ie) * (ax+ie)) + (ay * ay))\n\n\
adiffr = apath1 - apath2\n\nadlyr = adiffr / $SVEL\n\
adlyr = sqrt(adlyr * adlyr)\n\nasigl = asigl * aamplsig\n\
asigr = asigr * aamprsig\n\n", sizeof ( buf ) );
	buf [ sizeof ( buf ) - 1 ] = 0;
	target->writeF ( buf );

	strncpy ( buf, "asigld  vdelay asigl, adlyl*1000, 29\nasigrd vdelay \
asigr, adlyr*1000, 29\n\n\
alpath1 = asigl\nalpath2 = asigl\n\nkdiffl downsamp adiffl\nif kdiffl = 0 \
kgoto endxposl\n\nif kdiffl < 0 kgoto objectrightl\n\
alpath2 = asigld\nkgoto endxposl\n\nobjectrightl:\n\
alpath1 = asigld\nendxposl:\n\narpath1 = asigr\narpath2 = asigr\n\n\
kdiffr downsamp adiffr\n\
if kdiffr = 0 kgoto endxposr\n\nif kdiffr < 0 kgoto \
objectrightr\narpath2 = asigrd\nkgoto endxposr\n\n", sizeof ( buf ) );
	buf [ sizeof ( buf ) - 1 ] = 0;
	target->writeF ( buf );

	strncpy ( buf, "objectrightr:\narpath1 = asigrd\nendxposr:\n\nouts \
alpath1 * a2l, alpath2 * a3l\nouts arpath1 * a2r, arpath2 * a3r\n\ngarvbsig \
= 0\n\nskipreverb:\n\nendin\n\n; end ", sizeof ( buf ) );
	buf [ sizeof ( buf ) - 1 ] = 0;
	target->writeF ( buf );

	snprintf ( buf, sizeof ( buf ), "%s\n", orchFile );
	buf [ sizeof ( buf ) - 1 ] = 0;
	target->writeF ( buf );

	delete target;
}
