/*
	DWaveCmd.cc

	Will Eighteen, 2001: will@opaline.demon.co.uk
	Dynamic menu: code Xi Eighteen, 2001, implementation Will Eighteen, 2001
*/

#include "DWaveCmd.h"
#include "Application.h"
#include "AppWindow.h"
#include "MessageDialogManager.h"
#include <stdarg.h>
#include <Xm/Protocols.h>
#include <Xm/RowColumn.h>
#include <Xm/PushB.h>
#include <Xm/ToggleB.h>
#include <Xm/Label.h>


DWaveCmd::DWaveCmd ( const char *name, int active, Boolean quitOnSelect, DWaveCmdCallback callback,
					 void *clientData, char *menuLabelText, ... ) : NoUndoCmd ( name, active )
{
	_fixedString = TRUE;
	strncpy ( _name, name, sizeof ( _name ) );
	_name [ sizeof ( _name ) - 1 ] = 0;
	_callback = callback;
	_clientData = clientData;
	_menuItemList[0] = 0;
	_menuLabelText = menuLabelText;
	_quitOnSelect = quitOnSelect;
	_popupRealised = False;

	char *menuItemText;
	va_list menuList;
	_numMenuItems = 0;

	va_start ( menuList, menuLabelText );

	while ( va_arg ( menuList, char * ) )
		_numMenuItems++;	// copy this to storage + delete fn!

	va_end ( menuList );

	va_start ( menuList, menuLabelText );
	menuItemText = va_arg ( menuList, char * );
	_charArraySize = 0;

	for ( int n = 1; menuItemText; n++ )
	{
		char labelString [ strlen ( stripName ( menuItemText ) ) + 1 ];
				strcpy ( labelString, stripName ( menuItemText ) );

		if ( ( strlen ( _menuItemList ) + strlen ( labelString ) + 1 ) >= MENUSTORAGESPACE )
		{
			char msg[BUFSIZE];

			snprintf ( msg, sizeof ( msg ), "Menu item space required exceeds storage space\n\
%d specified in DWaveCmd.h", MENUSTORAGESPACE );
			msg [ sizeof ( msg ) - 1 ] = 0;
			messageDialogManager->post ( msg );
		}
		else
		{
			if ( strlen ( labelString ) > _charArraySize )
				_charArraySize = strlen ( labelString );
				
			strcat ( _menuItemList, labelString );
			strcat ( _menuItemList, "," );
		}

		menuItemText = va_arg ( menuList, char * );
	}

	va_end ( menuList );
}


DWaveCmd::DWaveCmd ( const char *name, int active, Boolean quitOnSelect, DWaveCmdCallback callback,
					 void *clientData, char *menuLabelText,
					 int numMenuItems,
					 int charArraySize,
					 char *itemArray ) : NoUndoCmd ( name, active )
{
	_fixedString = FALSE;
	strncpy ( _name, name, sizeof ( _name ) );
	_name [ sizeof ( _name ) - 1 ] = 0;
	_callback = callback;
	_clientData = clientData;
	_menuItemList[0] = 0;
	_menuLabelText = menuLabelText;
	_numMenuItems = numMenuItems;
	_charArraySize = charArraySize;
	_pItemArray = itemArray;
	_quitOnSelect = quitOnSelect;
	_popupRealised = False;
	
	for ( unsigned int item = 0; item < _numMenuItems; item++ )
	{
		char labelString [ strlen ( stripName ( ( char * ) &_pItemArray[ item * charArraySize ] ) ) + 1 ];
		strcpy ( labelString, stripName ( ( char * ) &_pItemArray[ item * charArraySize ] ) );

		if ( ( strlen ( _menuItemList ) + strlen ( labelString ) + 1 ) >= MENUSTORAGESPACE )
		{
			char msg[BUFSIZE];

			snprintf ( msg, sizeof ( msg ), "Menu item space required exceeds storage space\n\
%d allocated in DWaveCmd.h", MENUSTORAGESPACE );
			msg [ sizeof ( msg ) -1 ] = 0;
			messageDialogManager->post ( msg );
		}
		else
		{
			strcat ( _menuItemList, labelString );
			strcat ( _menuItemList, "," );
		}
	}
}


DWaveCmd::~DWaveCmd()
{
	if ( _popupRealised == True )
		quit();
}


void DWaveCmd::doCommand()
{
	unsigned int n, m;
	Arg args[3];
	Atom WM_DELETE_WINDOW;

	_popup = XtCreatePopupShell ( _name, transientShellWidgetClass,
									application->baseWidget(), NULL, 0 );

	_menu = XtVaCreateManagedWidget ( "DWaveMenu", xmRowColumnWidgetClass, _popup,
										XmNorientation, XmVERTICAL,
										XmNpacking, XmPACK_TIGHT,
										XmNnumColumns, _numMenuItems+2,
										NULL );

	_menuLabel = XtCreateManagedWidget ( _menuLabelText, xmLabelWidgetClass, _menu, NULL, 0 );

	m = 0;
	XtSetArg ( args[m], XmNtoggleMode, XmTOGGLE_BOOLEAN ); m++;
	XtSetArg ( args[m], XmNhighlightThickness, 0 ); m++;

	if ( _fixedString == TRUE )
	{
		unsigned int start = 0;
		unsigned int end = 0;
		unsigned int ptr;

		arrayOfPointersToMemBlocks = new waveWidgetSet * [ _numMenuItems ];

		for ( n = 1; n < ( _numMenuItems + 1 ); n++ )
		{
			_rc = XtVaCreateManagedWidget ( "RowColumn", xmRowColumnWidgetClass, _menu,
											XmNorientation, XmVERTICAL,
											XmNpacking, XmPACK_COLUMN,
											XmNnumColumns, 2, NULL );

			arrayOfPointersToMemBlocks[n-1] = new waveWidgetSet;
			arrayOfPointersToMemBlocks[n-1]->n = n;
			arrayOfPointersToMemBlocks[n-1]->waveMenu = this;

			while ( _menuItemList[end] != ',' )
				end++;

			char item [ end - start + 1 ];
			ptr = 0;

			while ( start < end )
			{
				item[ptr] = _menuItemList[start];
				start++;
				ptr++;
			}

			item[ptr++] = 0;
			end++;
			start = end;

			if ( n < _numMenuItems )
			{
				_toggle = XtCreateManagedWidget ( "toggle", xmToggleButtonWidgetClass, _rc, args, m );
				XtAddCallback( _toggle, XmNvalueChangedCallback, &DWaveCmd::opModeCallback, ( XtPointer ) arrayOfPointersToMemBlocks[n-1] );

				// set initial opMode from wave selection status
				if ( (*wavePak)[n-1].startSel == (*wavePak)[n-1].endSel )
					(*wavePak)[n-1].opMode = 0;
				else
					(*wavePak)[n-1].opMode = 1;
					
				if ( (*wavePak)[n-1].opMode == 0 )
				{
					XmString xmstr = XmStringCreateSimple ( "All" );
					XtVaSetValues ( _toggle, XmNlabelString, xmstr, NULL );
					XmStringFree ( xmstr );
					XtVaSetValues ( _toggle, XmNset, XmUNSET, NULL );
				}
				else
				{
					XmString xmstr = XmStringCreateSimple ( "Region" );
					XtVaSetValues ( _toggle, XmNlabelString, xmstr, NULL );
					XmStringFree ( xmstr );
					XtVaSetValues ( _toggle, XmNset, XmSET, NULL );
				}

				arrayOfPointersToMemBlocks[n-1]->toggle = ( Widget * ) _toggle;
			}
			
			char labelString [ strlen ( stripName ( item ) ) + 1 ];
			strcpy ( labelString, stripName ( item ) );

			_entry = XtVaCreateManagedWidget ( labelString,
												xmPushButtonWidgetClass, _rc, 
												NULL );

			XtAddCallback( _entry, XmNactivateCallback, &DWaveCmd::menuEntrySelectCallback, ( XtPointer ) arrayOfPointersToMemBlocks[n-1] );
		}
	}
	else
	{
		arrayOfPointersToMemBlocks = new waveWidgetSet * [ _numMenuItems ];

		for ( unsigned int item = 0; item < _numMenuItems; item++ )
		{
			_rc = XtVaCreateManagedWidget ( "RowColumn", xmRowColumnWidgetClass, _menu,
											XmNorientation, XmVERTICAL,
											XmNpacking, XmPACK_COLUMN,
											XmNnumColumns, 2, NULL );

			arrayOfPointersToMemBlocks[ item ] = new waveWidgetSet;
			arrayOfPointersToMemBlocks[ item ]->n = item + 1;
			arrayOfPointersToMemBlocks[ item ]->waveMenu = this;

			if ( item < ( _numMenuItems - 1 ) )
			{
				_toggle = XtCreateManagedWidget ( "toggle", xmToggleButtonWidgetClass, _rc, args, m );
				XtAddCallback ( _toggle, XmNvalueChangedCallback, &DWaveCmd::opModeCallback, ( XtPointer ) arrayOfPointersToMemBlocks[item] );

				if ( (*wavePak)[item].startSel == (*wavePak)[item].endSel )
					(*wavePak)[item].opMode = 0;
				else
					(*wavePak)[item].opMode = 1;

				if ( (*wavePak)[item].opMode == 0 )
				{
					XmString xmstr = XmStringCreateSimple ( "All" );
					XtVaSetValues ( _toggle, XmNlabelString, xmstr, NULL );
					XmStringFree ( xmstr );
					XtVaSetValues ( _toggle, XmNset, XmUNSET, NULL );
				}
				else
				{
					XmString xmstr = XmStringCreateSimple ( "Region" );
					XtVaSetValues ( _toggle, XmNlabelString, xmstr, NULL );
					XmStringFree ( xmstr );
					XtVaSetValues ( _toggle, XmNset, XmSET, NULL );
				}
				
				arrayOfPointersToMemBlocks [ item ]->toggle = ( Widget * ) _toggle;
			}

			char labelString [ strlen ( stripName ( ( char * ) &_pItemArray[ item * _charArraySize ] ) ) + 1 ];
			strcpy ( labelString, stripName ( ( char * ) &_pItemArray[ item * _charArraySize ] ) );

			_entry = XtVaCreateManagedWidget ( ( char * ) labelString,
								xmPushButtonWidgetClass, _rc,
								XmNheight, 10,
								NULL );

			XtAddCallback( _entry, XmNactivateCallback,
					&DWaveCmd::menuEntrySelectCallback,
					( XtPointer ) arrayOfPointersToMemBlocks[item] );
		}
	}
	
	// cater for WM 'X' pressed
	WM_DELETE_WINDOW = XmInternAtom ( XtDisplay ( _popup ), "WM_DELETE_WINDOW", False );
	XmAddWMProtocols ( _popup, &WM_DELETE_WINDOW, 1 );
	XmAddWMProtocolCallback ( _popup, WM_DELETE_WINDOW, clientMessageProc, ( XtPointer ) this );

	_popupRealised = True;
	XtManageChild ( _popup );
}


void DWaveCmd::clientMessageProc ( Widget, XtPointer clientData, XtPointer cbs )
{
	DWaveCmd *obj = ( DWaveCmd * ) clientData;

	obj->clientMessage ( ( XmAnyCallbackStruct * ) cbs );
	
}


void DWaveCmd::clientMessage ( XmAnyCallbackStruct *cbs )
{
	if ( cbs->reason != XmCR_CANCEL && cbs->reason != XmCR_OK )
		menuEntrySelect ( _numMenuItems );

}


void DWaveCmd::menuEntrySelectCallback( Widget, XtPointer clientData, XtPointer )
{
	waveWidgetSet *widgetPak = ( waveWidgetSet * ) clientData;
	DWaveCmd *managedMenu = widgetPak->waveMenu;

	managedMenu->menuEntrySelect ( widgetPak->n );
 }

// class member functions for callback functions

void DWaveCmd::menuEntrySelect ( unsigned int itemSelected )
{
	Boolean exit = False;
	char label [ _charArraySize + 1 ];

	if  ( itemSelected == 0 )
		label[0] = 0;
	else
	{
		strncpy ( label, itemToLabel ( itemSelected ), sizeof ( label ) );
		label [ sizeof ( label ) - 1 ] = 0;
	}

	if ( ( itemSelected == _numMenuItems ) || ( _quitOnSelect == True ) || ( itemSelected == 0 ) )
		exit = True;
	else
	{
		char wave [ strlen ( stripName ( (*wavePak)[itemSelected-1].pWave->name() ) ) + 1 ];
		strncpy ( wave, stripName ( (*wavePak)[itemSelected-1].pWave->name() ), sizeof ( wave ) );
		wave [ sizeof ( wave ) - 1 ] = 0;

		if ( strcmp ( label, wave ) != 0 ) 
		{
			if ( mixwaveResources->enableMessagePopups() == True )
			{
				char msg[60];
				strcpy ( msg, "Waves changed: please close and re-open wave control popup." );
				messageDialogManager->post ( msg );
			}

			return;
		}

		XmString xmstr1 = XmStringCreateSimple ( "Current wave: " );
		XmString xmstr2 = XmStringCreateSimple ( label );
		XmString xmstr3 = XmStringConcat ( xmstr1, xmstr2 );

		XtVaSetValues ( _menuLabel, XmNlabelString, xmstr3, NULL );

		XmStringFree ( xmstr1 );
		XmStringFree ( xmstr2 );
		XmStringFree ( xmstr3 );
	}

	setSelectedItem ( itemSelected );

	_callback ( _clientData, itemSelected, ( char * ) &label );	// also report back on 'quit selected'

	if ( exit == True )
		quit();
}


void DWaveCmd::quit()
{
	if ( _numMenuItems > 0 )
	{
		for ( unsigned int count = 0; count < _numMenuItems; count++ )
			delete arrayOfPointersToMemBlocks[ count ];

		delete[] arrayOfPointersToMemBlocks;
		arrayOfPointersToMemBlocks = 0;
		_numMenuItems = 0;
		_menuItemList[0] = 0;
	}

	if ( _popupRealised == True )
	{
		XtDestroyWidget ( _popup );
		_popupRealised = False;
	}
}


void DWaveCmd::opModeCallback ( Widget, XtPointer clientData, XtPointer callData )
{
	waveWidgetSet *widgetPak = ( waveWidgetSet * ) clientData;
	XmToggleButtonCallbackStruct *cbs = ( XmToggleButtonCallbackStruct * ) callData;

	DWaveCmd *managedMenu = widgetPak->waveMenu;
	unsigned int mode = cbs->set;
	managedMenu->opMode ( widgetPak, mode );
}


void DWaveCmd::opMode ( waveWidgetSet *widgetPak, unsigned int mode )
{
	// check the wave name and then check selection valid
	char label [ strlen ( itemToLabel ( widgetPak->n ) ) + 1 ];
	char name [ strlen ( stripName ( (*wavePak)[widgetPak->n-1].pWave->name() ) ) + 1 ];

	strcpy ( label, itemToLabel ( widgetPak->n ) );
	strcpy ( name, stripName ( (*wavePak)[widgetPak->n-1].pWave->name() ) );
	
	if ( strcmp ( label, name ) == 0 )
	{
		if ( (*wavePak)[widgetPak->n-1].startSel != (*wavePak)[widgetPak->n-1].endSel )
		{
			if ( mode == 0 )
			{	// override selected region in MixCmd
				XmString xmstr = XmStringCreateSimple ( "All" );
				XtVaSetValues ( *widgetPak->toggle, XmNlabelString, xmstr, NULL );
				XmStringFree ( xmstr );
			}
			else
			{
				XmString xmstr = XmStringCreateSimple ( "Region" );
				XtVaSetValues ( *widgetPak->toggle, XmNlabelString, xmstr, NULL );
				XmStringFree ( xmstr );
			}
			
			(*wavePak)[widgetPak->n-1].opMode = mode;
		}
		else
		{
			if ( mode == 1 )	// set selection requested but no sel. region in wave
			{
				if ( mixwaveResources->enableWarningPopups() == True )
				{
					char msg [ strlen ( (*wavePak)[widgetPak->n-1].pWave->name() ) + 36 ];
					
					snprintf ( msg, sizeof ( msg ), "Wave control: no selection made in\n%s",
								(*wavePak)[widgetPak->n-1].pWave->name() );
					msg [ sizeof ( msg ) - 1 ] = 0;
					messageDialogManager->post ( msg );
				}

				(*wavePak)[widgetPak->n-1].opMode = 0;
				XtVaSetValues ( *widgetPak->toggle, XmNset, XmUNSET, NULL );
			}
		}
	}
	else
	{
		if ( mixwaveResources->enableMessagePopups() == True )
		{
			char msg [ strlen ( ( char * ) (*wavePak)[widgetPak->n-1].pWave->name() ) +
						strlen ( ( char * ) itemToLabel ( widgetPak->n ) ) + 74 ];

			snprintf ( msg, sizeof ( msg ), "Menu item %s refers to wave\n%s.\n\
Please close and re-open Wave control popup.",
						( char * ) itemToLabel ( widgetPak->n ),
						( char * ) (*wavePak)[widgetPak->n-1].pWave->name() );
						
			msg [ sizeof ( msg ) - 1 ] = 0;
			messageDialogManager->post ( msg );
		}	
	}
}


char * DWaveCmd::stripName ( char *path )
{
	int i = strlen ( path );
	unsigned int j = 0;
	
	while  ( ( path[i] != '/' ) && ( i > -1 ) )
		i--;
	
	i++;
	
	while ( i < ( int ) strlen ( path ) )
	{
		if ( j == sizeof ( _fileName ) )
			i = strlen ( path );
		else
		{
			_fileName[j] = path[i];
			i++;
			j++;
		}
	}
	
	_fileName[j] = 0;
	return ( char * ) &_fileName;
}


char * DWaveCmd::itemToLabel ( unsigned int itemSelected )
{
	unsigned int ptr = 0;
	unsigned int n = 0;
	unsigned int item = 0;

	for ( n = 0; n < strlen ( _menuItemList ); n++ )
	{
		if ( _menuItemList[n] == ',' )
			item++;
			
		if ( item == ( itemSelected - 1 ) )
		{
			ptr = n;
			n = strlen ( _menuItemList );
		}
	}

	if ( ptr > 0 )
		ptr++;

	n = 0;

	while ( ( _menuItemList[ptr] != ',' ) && ( n < sizeof ( _label ) ) )
	{
		_label[n] = _menuItemList[ptr];
		n++;
		ptr++;
	}
	
	_label [ n ] = 0;
	return ( char * ) &_label;
}
