/*
	DataConformCmd.cc

	Will Eighteen, 2000: will@opaline.demon.co.uk
*/

#include "DataConformCmd.h"
#include "OpResources.h"
#include <FileAccess.h>
#include <math.h>


DataConformCmd::DataConformCmd ( char *name, int active, float *fltArray, int *intArray,
									long start, long end, unsigned int numPoints, float weighting,
									int dataType, char * dataFileName ) : BreakCmd ( name, active )
{
	_once = 0;
	_percentDone = 0;
	_processNum = 0;	// internal progress marker
	_fdata = fltArray;
	_idata = intArray;
	_numPoints = numPoints;
	_weighting = weighting;
	_startData = start;
	_endData = end;
	_dataType = dataType;
	_oldMax = _newMax = 0;
	_ptr = 0;
	_error = 0;
	
	_transformFile[0] = 0;
	_tf = NULL;
	_offsetInitialised = False;
	
	if ( numPoints > 0 )
	{
		if ( strlen ( dataFileName ) > sizeof ( _transformFile ) - 1 )
			_error = 1;	// name too long for buffer
		else
		{
			strncpy ( _transformFile, dataFileName, sizeof ( _transformFile ) );
			_transformFile [ sizeof ( _transformFile ) - 1 ] = 0;
			strncat ( _transformFile, "_tmp", sizeof ( _transformFile ) - strlen ( _transformFile ) - 1 );
			_transformFile [ sizeof ( _transformFile ) - 1 ] = 0;

			_tf = new FileAccess ( _transformFile, "w+" );
		}
	}
	
	if ( numPoints == 0 )
		_numProcesses = 2;
	else
		_numProcesses = 6;

	updateTimeout ( 100 );
}


DataConformCmd::~DataConformCmd()
{	
	if ( _tf != NULL )
		delete _tf;
}
	

void DataConformCmd::doCommand()
{
	if ( _error )	// constructor failed
	{
		_done = TRUE;
		return;
	}

	int percent, numItems;
	char text[40];
	long blockLength;
	long count;			
	long i, j;
	long double current;
	long double increment;
	float *fltArray;

	if ( !_once )
	{
		_processedSamples = 0;
		_start = _startData;
		
		if ( _dataType == 0 )
			_oldMax = _idata [ _start ];
		else
			_oldMax = _fdata [ _start ];

		if ( _oldMax < 0 )
			_oldMax *= -1;

		_ptr = _startData + 1;
		_once = 1;
	}

	if ( _dataType == 0 )	// integer - warning, only float handling tested (dataType 1)
	{
		if ( _processNum == 0 )
		{
			if ( _ptr < _endData )
			{
				blockLength = mixwaveResources->blockLength();

				if ( blockLength >  ( _endData - _ptr ) )
					blockLength = _endData - _ptr;

				for ( i = _ptr; i < _ptr + blockLength; i++ )
				{
					if ( sqrt (  _idata[i] * _idata[i] ) > _oldMax )
						_oldMax = sqrt ( _idata[i] * _idata[i] );

					if ( _idata [ i ] != _idata [ i - 1 ] )
					{
						_lastPos = i;
						current = _idata [ _start ];

						if ( ( i - _start ) > 0 )	// at least 2 numbers the same
						{
							increment = ( ( long double ) _idata [ i ] - ( long double ) _idata [ _start ] ) / ( i - _start );

							for ( j = 1; j < ( i - _start ); j++ )
							{
								current += increment;
								_idata [ _start + j ] = ( int ) ( current + 0.5 );
							}
						}

						_start = i;
					}
				}

				_ptr += blockLength;
				_processedSamples += blockLength;
			}

			if ( _ptr == _endData )
			{
				_ptr = _lastPos;
				_processNum++;
				_processedSamples = 0;
			}
		}	

		if ( _processNum == 1 )
		{
			if ( _lastPos != ( _endData - 1 ) )
			{
				current = _idata [ _lastPos ];
				increment = ( ( long double ) _idata [ _endData - 1 ] - ( long double ) current ) / ( _endData - _lastPos );

				if ( _ptr < _endData )
				{
					blockLength = mixwaveResources->blockLength();

					if ( blockLength > ( _endData - _ptr ) )
						blockLength = _endData - _ptr;

					for ( i = _ptr; i < _ptr + blockLength; i++ )
					{
						current += increment;
						_idata [ i ] = ( int ) ( current + 0.5 );
					}
				}

				_ptr += blockLength;

				if ( _ptr == _endData )
				{
					// we have linearly-interpolated data
					if ( _numPoints == 0 )	// curve reconstruction not required
					{
						_done = True;
						return;
					}

					if ( ( _endData - _startData ) < ( 2 * ( long ) _numPoints ) )
					{
						_error = 1;	// increase selection range or reduce numPoints
						return;
					}

					_ptr = _startData + ( long ) _numPoints;	// reset to data start + offset for datum reconstruction
					_processNum++;
					_processedSamples = 0;
				}
			}
			else
			{
				_ptr = _startData + ( long ) _numPoints;	// reset to data start + offset for datum reconstruction
				_processNum++;
				_processedSamples += blockLength;
			}
		}

		if ( _processNum == 2 )
		{
			_newMax = 0;
			i = 0;
			j = 0;

			if ( _ptr < ( _endData - ( long ) _numPoints ) )
			{
				blockLength = mixwaveResources->blockLength() / ( 2 * _numPoints + 1 );

				if ( _ptr > _endData - blockLength - ( long ) _numPoints + 1 )
					blockLength = _endData - _ptr - ( long ) _numPoints + 1;

				fltArray = new float [ blockLength ];
				_tf->elementSize ( sizeof ( float ) );
				_tf->numElements ( blockLength );	

				for ( count = _ptr; count < _ptr + blockLength; count++ )
				{
					current = 0;
					fltArray [ j ] = 0;

					for ( i = ( long ) -_numPoints; i <= ( long ) _numPoints; i ++ )	// do either side
						current += ( long double ) ( _idata [ count + i ] / ( _weighting * sqrt ( i * i ) + 1 ) );

					if ( sqrt ( current * current ) > _newMax )
						_newMax = sqrt ( current * current );

					fltArray [ j ] = ( float ) current;
					j++;
				}

				numItems = _tf->writeF ( ( void * ) fltArray );

				if ( numItems != blockLength )
					_error = 2;

				delete[] fltArray;

				if ( _error != 0 )
					return;
			}

			_ptr += blockLength;
			_processedSamples += blockLength;

			if ( _ptr == _endData - ( long ) _numPoints + 1 )
			{
				_scale = _oldMax / _newMax;

				if ( _scale > 1 )
					_scale = 1 / _scale;

				_tf->filePos ( 0 );
				_ptr = _startData + ( long ) _numPoints;
				_processNum++;
				_processedSamples = 0;
			}
		}

		if ( _processNum == 5 )
		{
			if ( _ptr < ( _endData - ( long ) _numPoints + 1 ) )
			{
				blockLength = mixwaveResources->blockLength();

				if ( blockLength > ( _endData - _ptr  - ( long ) _numPoints + 1 ) )
					blockLength = _endData - _ptr - ( long ) _numPoints + 1;

				fltArray = new float [ blockLength ];
				_tf->elementSize ( sizeof ( float ) );
				_tf->numElements ( blockLength );
				numItems = _tf->readF ( ( void * ) fltArray );
				j = 0;

				if ( numItems != blockLength )
				{
					_error = 3;
					delete[] fltArray;
					return;
				}

				for ( i = _ptr; i < _ptr + blockLength; i++ )
				{

					_idata [ i ] = ( int ) ( fltArray [ j ] + 0.5 );
					j++;
				}

				_ptr += blockLength;
				_processedSamples += blockLength;
				delete[] fltArray;
			}

			if ( _ptr == ( _endData - ( long ) _numPoints + 1 ) )
				_processNum++;			
		}
	}
	else
	{
		if ( _processNum == 0 )
		{
			if ( _ptr < _endData )
			{
				blockLength = mixwaveResources->blockLength();

				if ( blockLength >  ( _endData - _ptr ) )
					blockLength = _endData - _ptr;

				for ( i = _ptr; i < _ptr + blockLength; i++ )
				{
					if ( sqrt ( _fdata[i] * _fdata[i] ) > _oldMax )
						_oldMax = sqrt ( _fdata[i] * _fdata[i] );

					if ( _fdata [ i ] != _fdata [ i - 1 ] )
					{
						_lastPos = i;
						current = _fdata [ _start ];

						if ( ( i - _start ) > 0 )	// at least 2 numbers the same
						{
							increment = ( ( long double ) _fdata [ i ] - ( long double ) _fdata [ _start ] ) / ( i - _start );

							for ( j = 1; j < ( i - _start ); j++ )
							{
								current += increment;
								_fdata [ _start + j ] = ( float ) current;
							}
						}

						_start = i;
					}
				}

				_ptr += blockLength;
				_processedSamples += blockLength;
			}

			if ( _ptr == _endData )
			{
				_ptr = _lastPos;
				_processNum++;
				_processedSamples = 0;
			}
		}	

		if ( _processNum == 1 )
		{
			if ( _lastPos != ( _endData - 1 ) )
			{
				current = _fdata [ _lastPos ];
				increment = ( ( long double ) _fdata [ _endData - 1 ] - ( long double ) current ) / ( _endData - _lastPos );

				if ( _ptr < _endData )
				{
					blockLength = mixwaveResources->blockLength();

					if ( blockLength > ( _endData - _ptr ) )
						blockLength = _endData - _ptr;

					for ( i = _ptr; i < _ptr + blockLength; i++ )
					{
						current += increment;
						_fdata [ i ] = ( float ) current;
					}
				}

				_ptr += blockLength;

				if ( _ptr == _endData )
				{
					// we have linearly-interpolated data
					if ( _numPoints == 0 )	// curve reconstruction not required
					{
						_done = True;
						return;
					}

					if ( ( _endData - _startData ) < ( 2 * ( long ) _numPoints ) )
					{
						_error = 1;	// increase selection range or reduce numPoints
						return;
					}

					_ptr = _startData + ( long ) _numPoints;	// reset to data start + offset for datum reconstruction
					_processNum++;
					_processedSamples = 0;
				}
			}
			else
			{
				_ptr = _startData + ( long ) _numPoints;	// reset to data start + offset for datum reconstruction
				_processNum++;
				_processedSamples += blockLength;
			}
		}

		if ( _processNum == 2 )
		{
			_newMax = 0;
			i = 0;
			j = 0;

			if ( _ptr < ( _endData - ( long ) _numPoints ) )
			{
				blockLength = mixwaveResources->blockLength() / ( 2 * _numPoints + 1 );

				if ( _ptr > _endData - blockLength - ( long ) _numPoints + 1 )
					blockLength = _endData - _ptr - ( long ) _numPoints + 1;

				fltArray = new float [ blockLength ];
				_tf->elementSize ( sizeof ( float ) );
				_tf->numElements ( blockLength );	

				for ( count = _ptr; count < _ptr + blockLength; count++ )
				{
					current = 0;
					fltArray [ j ] = 0;

					for ( i = ( long ) -_numPoints; i <= ( long ) _numPoints; i ++ )	// do either side
						current += ( _fdata [ count + i ] / ( _weighting * sqrt(i*i) + 1 ) );

					if ( sqrt ( current * current ) > _newMax )
						_newMax = sqrt ( current * current );

					fltArray [ j ] = ( float ) current;
					j++;
				}

				numItems = _tf->writeF ( ( void * ) fltArray );

				if ( numItems != blockLength )
					_error = 2;

				delete[] fltArray;

				if ( _error != 0 )
					return;
			}

			_ptr += blockLength;
			_processedSamples += blockLength;

			if ( _ptr == _endData - ( long ) _numPoints + 1 )
			{
				_scale = _oldMax / _newMax;

				if ( _scale > 1 )
					_scale = 1 / _scale;

				_tf->filePos ( 0 );
				_ptr = _startData + ( long ) _numPoints;
				_processNum++;
				_processedSamples = 0;
			}
		}

		if ( _processNum == 5 )
		{
			if ( _ptr < ( _endData - ( long ) _numPoints + 1 ) )
			{
				blockLength = mixwaveResources->blockLength();

				if ( blockLength > ( _endData - _ptr  - ( long ) _numPoints + 1 ) )
					blockLength = _endData - _ptr - ( long ) _numPoints + 1;

				fltArray = new float [ blockLength ];
				_tf->elementSize ( sizeof ( float ) );
				_tf->numElements ( blockLength );
				numItems = _tf->readF ( ( void * ) fltArray );
				j = 0;

				if ( numItems != blockLength )
				{
					_error = 3;
					delete[] fltArray;
					return;
				}

				for ( i = _ptr; i < _ptr + blockLength; i++ )
				{

					_fdata [ i ] = fltArray [ j ];
					j++;
				}

				_ptr += blockLength;
				_processedSamples += blockLength;
				delete[] fltArray;
			}

			if ( _ptr == ( _endData - ( long ) _numPoints + 1 ) )
				_processNum++;			
		}
	}

	if ( _processNum == 3 )
	{
		if ( _ptr < _endData - ( long ) _numPoints + 1 )
		{
			blockLength = mixwaveResources->blockLength() / ( 2 * _numPoints + 1 );

			if ( blockLength > ( _endData - _ptr - ( long ) _numPoints + 1 ) )
				blockLength = _endData - _ptr - ( long ) _numPoints + 1;

			fltArray = new float [ blockLength ];
			_tf->elementSize ( sizeof ( float ) );
			_tf->numElements ( blockLength );
			i = _tf->filePos();
			numItems = _tf->readF ( ( void * ) fltArray );

			if ( numItems != blockLength )
			{
				_error = 3;
				delete[] fltArray;
				return;
			}

			for ( j = 0; j < blockLength; j++ )
				fltArray [ j ] *= _scale;

			_tf->filePos ( ( unsigned int ) i );
			numItems = _tf->writeF ( ( void * ) fltArray );

			if ( numItems != blockLength )
			{
				_error = 2;
				delete[] fltArray;
				return;
			}

			if ( _offsetInitialised == False )
			{
				if ( _dataType == 0 )
					_offset = (long double ) _idata [ _startData + _numPoints ] - fltArray [ 0 ];
				else
					_offset = _fdata [ _startData + _numPoints ] - fltArray [ 0 ];

				_offsetInitialised = True;
			}

			delete[] fltArray;
			_ptr += blockLength;
			_processedSamples += blockLength;
		}

		if ( _ptr == ( _endData - ( long ) _numPoints + 1 ) )
		{
			_tf->filePos ( 0 );
			_ptr = _startData + ( long ) _numPoints;
			_processNum++;
			_processedSamples = 0;
		}
	}

	if ( _processNum == 4 )
	{
		if ( _ptr < _endData - ( long ) _numPoints + 1 )
		{
			blockLength = mixwaveResources->blockLength() / ( 2 * _numPoints + 1 );

			if ( blockLength > ( _endData - _ptr - ( long ) _numPoints + 1 ) )
				blockLength = _endData - _ptr - ( long ) _numPoints + 1;

			fltArray = new float [ blockLength ];
			_tf->elementSize ( sizeof ( float ) );
			_tf->numElements ( blockLength );
			i = _tf->filePos();
			numItems = _tf->readF ( ( void * ) fltArray );

			if ( numItems != blockLength )
			{
				_error = 3;
				delete[] fltArray;
				return;
			}

			for ( j = 0; j < blockLength; j++ )
				fltArray [ j ] +=  ( float ) _offset;

			_tf->filePos ( ( unsigned int ) i );
			numItems = _tf->writeF ( ( void * ) fltArray );

			if ( numItems != blockLength )
			{
				_error = 2;
				delete[] fltArray;
				return;
			}

			delete[] fltArray;
			_ptr += blockLength;
			_processedSamples += blockLength;
		}

		if ( _ptr == ( _endData - ( long ) _numPoints + 1 ) )
		{
			_tf->filePos ( 0 );
			_ptr = _startData + ( long ) _numPoints;
			_processNum++;
			_processedSamples = 0;
		}
	}

	if ( _processNum == 6 )
	{
		delete _tf;
		_tf = NULL;
		_done = True;
	}
	else
	{
		percent = ( int ) ( ( float ) _processedSamples / ( float ) ( _endData - _startData + 1 ) * 100 );
		
		if ( _percentDone != percent )
		{
			_percentDone = percent;
			snprintf(text, sizeof ( text ), "%d%%\nprocess %u/%d", _percentDone, _processNum + 1, _numProcesses );
			text [ sizeof ( text ) - 1 ] = 0;
			updateMessage ( text );
		}
	}
}
