/*
	MixCmd.h - mix down mono input .wav files to stereo output

	Will Eighteen, 2001: will@opaline.demon.co.uk
*/

#ifndef MIXCMD_H
#define MIXCMD_H


#ifndef BREAK
#define BREAK
#include <BreakCmd.h>
#endif


class Wave;

class MixCmd : public BreakCmd
{
	public:
		MixCmd ( char *, int, char * );
		virtual const char *const className() { return ( "MixCmd" ); }		
		~MixCmd();
		char *mixName() { return _mixFileName; }
		char *error() { return _error; }
		unsigned long maxMixBits() { return maxBitSet ( _mixMagnitude ); }
		unsigned long lostBits() { return ( _lostBits ); }
		
	protected:
		void doCommand();	/* mixdown */

	private:
		Boolean allocMixBlock ( long );
		void freeMixBlock ( void );
		unsigned long maxBitSet( unsigned long );
		void loadGainArray ( unsigned int, unsigned long );
		void loadPanArray ( unsigned int, unsigned long );
		long *mix ( long );
		
		int preMix ( void );
		void postMix ( void );
		int initMixPakPointers ( void );
		void reInitWavePointers ( void );
		Boolean _primaryMixDone;
		long _timeout;
		int _percentDone;
		unsigned long _writeSamples;
		unsigned long _processedSamples;
		unsigned long _mixMagnitude;
		unsigned long _sampleCount;
		long *_pMixBlock;
		float *_gainArray;
		float *_panArray;
		unsigned long _mixBits;
		unsigned int _bytesPerSample;
		int _once;
		long _numWaves;
		long _currentWaveNum;
		long _start, _end;
		char *_oData;
		char *_lostData;
		Wave *_pMixed;
		char *_mixFileName;
		long _mixBlockLength;
		long *_mixData;
		float _halfPi;
		long _mixBytes, _maxDataLength;
		unsigned long _lostBits;
		char _error[1024];
};

#endif

/*
  Notes:
  
  If the WaveCtrlCmd popup ('Options/Wave control') isn't up, the mix may not
  be what you expect, since it relies on the last toggle settings when the
  popup was active, a wave view was active _and_ the 'region' toggle button
  for that wave was clicked. In short, if you do a 'load wave' or 'state load'
  and then select a viewed wave region, then do 'File/Mixdown', you won't get
  the region-only mixdown; rather, the entire wave will be used. Selected range
  values aren't transferred to the mixdown unless wave control is active.
*/
